#ifndef VFILELIST_H
#define VFILELIST_H

#include <QWidget>
#include <QJsonObject>
#include <QFileInfo>
#include <QDir>
#include "vnotebook.h"
#include "vconstants.h"

class QAction;
class VNote;
class QListWidget;
class QListWidgetItem;
class QPushButton;
class VEditArea;

class VFileList : public QWidget
{
    Q_OBJECT
public:
    explicit VFileList(VNote *vnote, QWidget *parent = 0);
    bool importFile(const QString &name);
    inline void setEditArea(VEditArea *editArea);
    void fileInfo(const QString &p_notebook, const QString &p_relativePath);
    void deleteFile(const QString &p_notebook, const QString &p_relativePath);

signals:
    void fileClicked(QJsonObject fileJson);
    void fileDeleted(QJsonObject fileJson);
    void fileCreated(QJsonObject fileJson);
    void fileRenamed(const QString &p_srcNotebook, const QString &p_srcRelativePath,
                     const QString &p_destNotebook, const QString &p_destRelativePath);
    void directoryChanged(const QString &notebook, const QString &relativePath);

private slots:
    void contextMenuRequested(QPoint pos);
    void handleItemClicked(QListWidgetItem *currentItem);
    void curFileInfo();
    void deleteCurFile();
    void copySelectedFiles(bool p_isCut = false);
    void cutSelectedFiles();
    void pasteFilesInCurDir();

public slots:
    void setDirectory(QJsonObject dirJson);
    void handleNotebookRenamed(const QVector<VNotebook> &notebooks, const QString &oldName,
                               const QString &newName);
    void handleDirectoryRenamed(const QString &notebook, const QString &oldRelativePath,
                                const QString &newRelativePath);
    void newFile();

private:
    void setupUI();
    void updateFileList();
    QListWidgetItem *insertFileListItem(QJsonObject fileJson, bool atFront = false);
    void removeFileListItem(QListWidgetItem *item);
    void initActions();
    bool isConflictNameWithExisting(const QString &name);
    QListWidgetItem *createFileAndUpdateList(const QString &name);
    void deleteFileAndUpdateList(const QString &p_notebook,
                                 const QString &p_relativePath);
    void clearDirectoryInfo();
    void convertFileType(const QString &notebook, const QString &fileRelativePath,
                         DocType oldType, DocType newType);
    QListWidgetItem *findItem(const QString &p_notebook, const QString &p_relativePath);
    void deleteLocalImages(const QString &filePath);
    void copyFileInfoToClipboard(const QJsonArray &p_files, bool p_isCut);
    void pasteFiles(const QString &p_notebook, const QString &p_dirRelativePath);
    bool copyFile(const QString &p_srcNotebook, const QString &p_srcRelativePath,
                  const QString &p_destNotebook, const QString &p_destRelativePath,
                  bool p_isCut);
    int removeFileInConfig(const QString &p_filePath);
    bool addFileInConfig(const QString &p_filePath, int p_index);
    QJsonObject readFileInConfig(const QString &p_filePath);

    VNote *vnote;
    QString notebook;
    // Current directory's relative path
    QString relativePath;
    // Used for cache
    QString rootPath;

    VEditArea *editArea;

    QListWidget *fileList;

    // Actions
    QAction *newFileAct;
    QAction *deleteFileAct;
    QAction *fileInfoAct;
    QAction *copyAct;
    QAction *cutAct;
    QAction *pasteAct;
};

inline void VFileList::setEditArea(VEditArea *editArea)
{
    this->editArea = editArea;
}

#endif // VFILELIST_H
